<?php


namespace App\Http\Controllers\Api;
use App\Http\Params\ProblemParam;
use App\Models\User;
use App\Models\UserInfoModel;
use App\Models\UserInviteLog;
use App\Models\UserVipLogModel;
use App\Models\VipConfig;
use App\Models\VipModel;
use App\Services\UserService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use PHPUnit\Util\Exception;

class UserController extends Controller
{
    protected $userService;

    public function __construct(){
        $this->userService = new UserService();
    }

    /**
     * 设置密码
     */
    public function setpass(Request $request){
        $validator = Validator::make($request->all(), [
            'password' => 'required|string',
        ],[
            'password.required'=>"密码必须",
        ]);
        if ($validator->fails()) {
            return $this->response()->errorForbidden($validator->messages()->first());
        }
        $user = auth('api')->user();
        $user->password = $request->password;
        if($user->save()){
            return response()->json(['message'=>"设置成功"]);
        }else{
            return $this->response->errorForbidden("设置失败");
        }
    }

    /**
     * 选择性别
     */
    public function checksex(Request $request){
        $user = auth('api')->user();
        $validator = Validator::make($request->all(), [
            'sex' => 'required'
        ], [
            'mobile.required'=>"性别必须",
        ]);
        if ($validator->fails()) {
            return $this->response()->errorForbidden($validator->messages()->first());
        }
        //dd($user);
        $user->sex = $request->sex;
        if (!$user->save()) {
            return $this->response->errorForbidden('设置性别失败');
        }
        return response()->json(['message'=>"设置成功"]);
    }

    /**
     * 设置资料（初次登录）
     */
    public function setinfo(Request $request){
        $user = auth('api')->user();

        DB::beginTransaction();
        try {
            $validator = Validator::make($request->all(), [
                'nickname' => 'required|between:2,10',
                'birthday' => 'required',
                'avatar' => 'required',
            ], [
                'nickname.required'=>"昵称不能为空",
                'nickname.between'=>"昵称长度必须在2~10之间",
                'birthday.required'=>"生日不能为空",
                'avatar.required'=>"头像不能为空",
            ]);
            if ($validator->fails()) {
                throw new Exception($validator->messages()->first());
            }

            if(UserInfoModel::where(['nickname'=>$request->nickname])->where('user_id', '!=' , $user->id)->first()){
                throw new Exception("昵称已被使用");
            }
            //邀请码设置
            $pid = 0;
            if(isset($request->ycode) && $request->ycode!=""){
                if(!$puser = User::where(['ycode'=>$request->ycode])->first()){
                    throw new Exception("邀请码不存在");
                }
                $pid = $puser->id;

                //赠送会员天数
                UserInviteLog::query()->create([
                    'user_id'=>$pid,
                    'invite_id'=>$user->id,
                    'day'=>1,
                    'status'=>0,
                ]);
            }

            UserInfoModel::query()->where('user_id',$user->id)->update([
                'avatar' =>$request->avatar,
                'nickname' =>$request->nickname,
                'birthday' =>$request->birthday,
            ]);
            if($pid!=0){
                $user->pid = $pid;
                $user->save();
            }
            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return $this->response()->errorForbidden($exception->getMessage());
        }
        return response()->json(['message'=>"设置成功"]);
    }

    /**
     * 获取个人资料
     */
    public function getinfo(){
        $user = auth('api')->user();
        $userinfo = UserInfoModel::where('user_id', $user->id)->first();

        $userinfo['hobby'] = explode(',',$userinfo['hobby']);
        $userinfo['photo'] = json_decode($userinfo['photo'],true);
        $userinfo['video'] = json_decode($userinfo['video'],true);
        $userinfo['like_num'] = $user->like_num;
        $userinfo['like_me_num'] = $user->like_me_num;
        $userinfo['look_num'] = $user->look_num;
        return response()->json($userinfo);
    }


    /**
     * 修改个人资料
     */
    public function updateinfo(Request $request){
        $user = auth('api')->user();
        $validator = Validator::make($request->all(), [
            'nickname' => 'required',
            'height' => 'integer',
            'weight' => 'integer',
        ], [
            'nickname.required'=>"昵称必须",
            'height.integer'=>"身高为整数",
            'weight.integer'=>"体重为整数",
        ]);
        if ($validator->fails()) {
            return $this->response()->errorForbidden($validator->messages()->first());
        }
        $userinfo = UserInfoModel::where('user_id', $user->id)->first();
        if($request->nickname!=$userinfo->nickname){
            if(UserInfoModel::where(['nickname'=>$request->nickname])->first()){
                return $this->response->errorForbidden('昵称已被使用');
            }
        }

        if(UserInfoModel::where('user_id', $user->id)->update($request->input())){
            return response()->json(['message'=>"修改成功"]);
        }else{
            return $this->response->errorForbidden("修改失败");
        }
    }

    /**
     * 修改个人更多介绍
     */
    public function updateext(Request $request){
        $user = auth('api')->user();
        if(UserInfoModel::where('user_id', $user->id)->update($request->input())){
            return response()->json(['message'=>"修改成功"]);
        }else{
            return $this->response->errorForbidden("修改失败");
        }
    }

    /**
     * 获取照片和视频
     */
    public function get_photo(){
        try {
            $user = auth('api')->user();
            $userinfo = DB::table('users_info')->where(['user_id'=>$user->id])->select('photo','video')->first();
            $userinfo->photo = !empty($userinfo->photo)?json_decode($userinfo->photo,true):[];
            $userinfo->video = !empty($userinfo->video)?json_decode($userinfo->video,true):[];
        }catch (\Exception $exception){
            return $this->response->errorForbidden($exception->getMessage());
        }
        return response()->json($userinfo);
    }

    /**
     * 上传照片或者视频
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|void
     */
        public function upload_file(Request $request){
        try {
            if($request->post('url')==''){
                throw new Exception("请选择上传内容");
            }
            $user = auth('api')->user();
            $userinfo = UserInfoModel::query()->where('user_id',$user->id)->first();
            if($request->type==1){
                $new_arr = [
                    "url"=>$request->post('url'),
                    "state"=>$request->post('state',0)
                ];
                //图片
                $user_photo = json_decode($userinfo->photo,true);
                $user_photo []= $new_arr;
                $userinfo->photo = json_encode($user_photo);
            }else{
                //视频
                $user_video = json_decode($userinfo->video,true);
                $user_video []= $request->post('url');
                $userinfo->video = json_encode($user_video);
            }
            $userinfo->save();
        }catch (\Exception $exception){
            return $this->response->errorForbidden($exception->getMessage());
        }
        return response()->json(['message'=>'上传成功']);
    }



    /**
     * 获取VIP
     */
    public function get_vip(){
        try {
            $res['list'] = VipModel::query()->orderBy('id','asc')->get();
            $res['rights'] = json_decode(VipConfig::query()->where(['id'=>2])->value('rights'),true);
            $config = config("filesystems.disks.oss");
            foreach ($res['rights'] as $k=>$v){
                $res['rights'][$k]['img_url'] = "https://".$config['bucket'].'.'.$config['endpoint'].'/'.$v['img_url'];
            }
        }catch (\Exception $exception){
            return $this->response->errorForbidden($exception->getMessage());
        }
        return response()->json($res);
    }

    /**
     * 获取当前vip信息
     */
    public function get_vip_info(){
        try {
            $user = auth('api')->user();

            $user_info = UserInfoModel::query()->where('user_id',$user->id)->first();
            $res['avatar'] = $user_info->avatar;
            $res['nickname'] = $user_info->nickname;
            $res['is_vip'] = $user->is_vip;

            if($user->is_vip>0){
                $info = UserVipLogModel::query()->where(['user_id'=>$user->id])->first();
                $res['end_day'] = date("Y-m-d",strtotime($info['end_day']));
            }else{
                $res['end_day'] = "";
            }

        }catch (\Exception $exception){
            return $this->response->errorForbidden($exception->getMessage());
        }
        return response()->json($res);
    }

    /**
     * 购买/续费vip
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|void
     */
    public function buy_vip(Request $request){
        try {
            $user = auth('api')->user();
            $param['id'] = $request->id;  //vip id
            $param['user_id'] = $user->id;
            $param['payment'] = $request->post('payment',1); //支付方式 1微信  2支付宝
            $res = $this->userService->buy_vip($param);
            //throw new Exception(json_encode($res));
        }catch (\Exception $exception){
            return $this->response->errorForbidden($exception->getMessage());
        }
        return response()->json($res);
    }


    /**
     * 问题反馈
     * @param Request $request
     * @return \Dingo\Api\Http\Response|void
     */
    public function problem(Request $request){
        try {
            $user = auth('api')->user();
            $param = new ProblemParam();
            $param->user_id = $user->id;
            $param->content = $request->post('content');
            $param->img_url = $request->post('img_url');
            $param->status = 0;
            $this->userService->problem($param);
        }catch (\Exception $exception){
            return $this->response->errorForbidden($exception->getMessage());
        }
        return response()->json(['message'=>"提交成功"]);
    }

    /**
     * 看过我
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|void
     */
    public function looked_me(Request $request){
        try {
            $user = auth('api')->user();
            $param['user_id'] = $user->id;
            $res =$this->userService->looked_me($param);
        }catch (\Exception $exception){
            return $this->response->errorForbidden($exception->getMessage());
        }
        return response()->json($res);
    }

    /**
     * 人脸对比
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|void
     */
    public function check_auth(Request $request){
        $apiFace = new \AipFace(env('BAI_DU_YUN_APP_ID'),env('BAI_DU_YUN_API_KEY'),env('BAI_DU_YUN_SECRET_KEY'));
        try {
           $images = array(
//                array(
//                    'image' => base64_encode(file_get_contents('https://zhengda.oss-accelerate.aliyuncs.com/tinymce/images/dcdc14c5987ebbb233a6232264bb80e260e7ef418313d.jpg')),
//                    'image_type' => 'BASE64',
////                    'liveness_control'=>'HIGH',
//                ),
                array(
                    'image' => base64_encode(file_get_contents('https://zhengda.oss-accelerate.aliyuncs.com/tinymce/images/cea206a2046b895f253445f35794226860e7ef561e3ac.jpg')),
                    'image_type' => 'BASE64',
                    'liveness_control'=>'NORMAL',
                ),
               array(
                    'image' => base64_encode(file_get_contents('https://zhengda.oss-accelerate.aliyuncs.com/tinymce/images/cbc1fe6ae109bd0d11b32d066dcd4d7f60e7f1ee5d624.jpg')),
                    'image_type' => 'BASE64',
//                    'liveness_control'=>'NORMAL',
                ),
            );
            $res = $apiFace->match($images);
        }catch (\Exception $exception){
            return $this->response->errorForbidden($exception->getMessage());
        }
        return response()->json($res);
    }

    /**
     * 黑名单
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|void
     */
    public function black_list(Request $request){
        try {
            $res = $this->userService->black_list();
        }catch (\Exception $exception){
            return $this->response->errorForbidden($exception->getMessage());
        }
        return response()->json($res);
    }

    /**
     * 移除黑名单
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|void
     */
    public function del_black(Request $request){
        try {
            $this->userService->del_black($request->id);
        }catch (\Exception $exception){
            return $this->response->errorForbidden($exception->getMessage());
        }
        return response()->json(['message'=>'操作成功']);
    }

    /**
     * 设置在线状态和通知开关
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|void
     */
    public function online_status(Request $request){
        try {
            if($request->type=='get'){
                $user = auth('api')->user();
                $res = User::query()->find($user->id,['online','notice_status']);
            }else{
                $res=$this->userService->online_status($request);
            }

        }catch (\Exception $exception){
            return $this->response->errorForbidden($exception->getMessage());
        }
        return response()->json(['message'=>'操作成功','data'=>$res]);
    }

    /**
     * 获取邀请福利信息
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|void
     */
    public function invite_info(Request $request){
        try {
            $res=$this->userService->invite_info($request);
        }catch (\Exception $exception){
            return $this->response->errorForbidden($exception->getMessage());
        }
        return response()->json($res);
    }

    /**
     * 获取邀请人员列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|void
     */
    public function get_invite_list(){
        try {
            $res=$this->userService->get_invite_list();
        }catch (\Exception $exception){
            return $this->response->errorForbidden($exception->getMessage());
        }
        return response()->json($res);
    }

    /**
     * 领取会员天数
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|void
     */
    public function receive_day(){
        DB::beginTransaction();
        try {
            $this->userService->receive_day();
            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return $this->response->errorForbidden($exception->getMessage());
        }
        return response()->json(['message'=>'操作成功']);
    }

    /**
     * 设置手势锁
     */
    public function lock_pass(){

    }


}
